import numpy as np
import scipy.stats as ss
import scipy.optimize as so
import matplotlib.pyplot as plt

N =  1000
theta0 = 0.7

def check(v, L, R):
    if L <= v and v <= R:
        return 1
    else:
        return 0
        
count_dict = {}
for i in np.arange(1, 100, 1):
    count_dict[i] = 0
    
n_simu = 10000

for t in range(n_simu):
    if t % 100 == 0:
        print(f'{t}/{n_simu}')
    k = ss.binom.rvs(N, theta0)
    for i in np.arange(1, 100, 1):
        L = (100.0 - i) / 2
        R = 100.0 - L
        v = ss.beta.ppf([L/100, R/100], k+1, N-k+1) 
        pL, pR = v[0], v[1]  
        count_dict[i] += check(theta0, pL, pR)
    
counts = []
for i in count_dict:
    counts.append(count_dict[i])
prop = np.array(counts) / n_simu


#
#   Confidence Interval for Binomial Distribution
#
class U_cum_bin():
    def __init__(self, N, k, a):
        self.N = N
        self.k = k
        self.a = a
        
    def f(self, p):
        return ss.binom.cdf(self.k, self.N, p) - self.a/2
    
def U_boundary(N, k, a):
    if k == N:
        v = 1.0
    else:
        func = U_cum_bin(N, k, a).f
        v = so.bisect(func, 0, 1)
    return v

class L_cum_bin():
    def __init__(self, N, k, a):
        self.N = N
        self.k = k
        self.a = a
        
    def f(self, p):
        return 1.0 - ss.binom.cdf(self.k - 1, self.N, p) - self.a/2
    
def L_boundary(N, k, a):
    if k == 0:
        v = 0.0
    else:
        func = L_cum_bin(N, k, a).f
        v = so.bisect(func, 0, 1)
    return v

prop_L = []
prop_U = []
for i, v in enumerate(counts):
    pL = L_boundary(n_simu, v, 0.05)
    pU = U_boundary(n_simu, v, 0.05)
    prop_L.append(pL)
    prop_U.append(pU)
            
#print(prop)
x_coord = [i for i in range(1, 100, 1)]
plt.plot(x_coord, prop, label = 'prop')
plt.plot(x_coord, prop_L, c = 'g', lw = 1)
plt.plot(x_coord, prop_U, c = 'g', lw = 1)
plt.fill_between(x_coord, prop_L, prop_U, color = 'g', alpha = 0.3, label = f'95%CI, n_simu = {n_simu}')
plt.plot([1, 99], [0.01, 0.99], c = 'r', label = 'x = y')
plt.xlabel(r'(1-$\alpha$)100%CI', fontsize = 16)
plt.ylabel(r'Prop($\theta_0\in$CI)', fontsize = 14)
plt.title(f'N = {N}, theta = {theta0}', fontsize = 20)
plt.legend()
plt.savefig('FigBinN1000.png')
plt.show()
