import numpy as np
import scipy.stats as ss
import scipy.optimize as so
import pystan
import pickle
import tkinter as tk
import matplotlib.pyplot as plt

sm = pystan.StanModel(file = 'NormalModel.stan')
with open('sm.pkl', 'wb') as f:
    pickle.dump(sm, f)

sample_size = 100  

def check(v, L, R):
    if L <= v and v <= R:
        return 1
    else:
        return 0
   
    
count_dict = {}
countsgm_dict = {}
for i in range(1, 100, 1):
    count_dict[i] = 0
    countsgm_dict[i] = 0
    
print(count_dict.keys())

n_simu = 2000

root = tk.Tk()
root.attributes('-topmost', True)
cnvs = tk.Canvas(root, bg = '#ffffff', width = 400, height = 300)
cnvs.pack()
cnvs.create_text(200, 150, text = 'Simulation started.',font = ('Gothic', 32), tags = 'mytext') 
cnvs.update()

for t in range(n_simu):
    cnvs.itemconfig('mytext', text = f't = {t+1}/{n_simu}')
    cnvs.update()
    X = ss.norm(0, 1).rvs(size = sample_size)
    with open('sm.pkl', 'rb') as f:
        sm = pickle.load(f)
    fit = sm.sampling(data = {'N':len(X), 'X':X}, iter = 10000, n_jobs = 1)
   
    for i in range(1, 100, 1):
        L = (100.0 - i) / 2
        R = 100.0 - L
        pL, pR = np.percentile(fit['mu'], [L, R])
        count_dict[i] += check(0.0, pL, pR)
        pL, pR = np.percentile(fit['sgm'], [L, R])
        countsgm_dict[i] += check(1.0, pL, pR)
    
root.destroy()
    
print(count_dict)
print(countsgm_dict)
counts = []
counts_sgm = []
for i in count_dict:
    counts.append(count_dict[i])
prop = np.array(counts) / n_simu
for i in countsgm_dict:
    counts_sgm.append(countsgm_dict[i])
propsgm = np.array(counts_sgm) / n_simu

root = tk.Tk()
root.attributes('-topmost', True)
cnvs = tk.Canvas(root, bg = '#ffffff', width = 400, height = 300)
cnvs.pack()
cnvs.create_text(200, 150, text = 'Simulation started.',font = ('Gothic', 32), tags = 'mytext') 
cnvs.update()


#
#   Confidence Interval for Binomial Distribution
#
class U_cum_bin():
    def __init__(self, N, k, a):
        self.N = N
        self.k = k
        self.a = a
        
    def f(self, p):
        return ss.binom.cdf(self.k, self.N, p) - self.a/2
    
def U_boundary(N, k, a):
    if k == N:
        v = 1.0
    else:
        func = U_cum_bin(N, k, a).f
        v = so.bisect(func, 0, 1)
    return v

class L_cum_bin():
    def __init__(self, N, k, a):
        self.N = N
        self.k = k
        self.a = a
        
    def f(self, p):
        return 1.0 - ss.binom.cdf(self.k - 1, self.N, p) - self.a/2
    
def L_boundary(N, k, a):
    if k == 0:
        v = 0.0
    else:
        func = L_cum_bin(N, k, a).f
        v = so.bisect(func, 0, 1)
    return v

prop_L = []
prop_U = []
propsgm_L = []
propsgm_U = []
for i, v in enumerate(counts):
    cnvs.itemconfig('mytext', text = f'mu: i = {i}/{len(counts)}')
    cnvs.update()
     
    pL = L_boundary(n_simu, v, 0.05)
    pU = U_boundary(n_simu, v, 0.05)
    prop_L.append(pL)
    prop_U.append(pU)
    
for i, v in enumerate(counts_sgm):
    cnvs.itemconfig('mytext', text = f'sgm: i = {i}/{len(counts)}')
    cnvs.update()
        
    pL = L_boundary(n_simu, v, 0.05)
    pU = U_boundary(n_simu, v, 0.05)
    propsgm_L.append(pL)
    propsgm_U.append(pU)
        
root.destroy()

           
print(prop)
x_coord = [i for i in range(1, 100, 1)]
plt.plot(x_coord, prop, label = 'prop')
plt.plot(x_coord, prop_L, c = 'g', lw = 1)
plt.plot(x_coord, prop_U, c = 'g', lw = 1)
plt.fill_between(x_coord, prop_L, prop_U, color = 'g', alpha = 0.3, label = f'95%CI, n_simu = {n_simu}')
plt.plot([1, 99], [0.01, 0.99], c = 'r', label = 'x = y')
plt.xlabel(r'(1-$\alpha$)100%CI', fontsize = 16)
plt.ylabel(r'Prop($\mu_0\in$CI)', fontsize = 14)
plt.title(f'N = {sample_size}, mu', fontsize = 20)
plt.legend()
plt.savefig(f'FigNormMuN{sample_size}.png')
plt.show()
    
    
print(propsgm)
x_coord = [i for i in range(1, 100, 1)]
plt.plot(x_coord, propsgm, label = 'prop')
plt.plot(x_coord, propsgm_L, c = 'g', lw = 1)
plt.plot(x_coord, propsgm_U, c = 'g', lw = 1)
plt.fill_between(x_coord, propsgm_L, propsgm_U, color = 'g', alpha = 0.3, label = f'95%CI, n_simu = {n_simu}')
plt.plot([1, 99], [0.01, 0.99], c = 'r', label = 'x = y')
plt.xlabel(r'(1-$\alpha$)100%CI', fontsize = 16)
plt.ylabel(r'Prop($\sigma_0\in$CI)', fontsize = 14)
plt.title(f'N = {sample_size}, sigma', fontsize = 20)
plt.legend()
plt.savefig(f'FigNormSgmN{sample_size}.png')
plt.show()
 
